'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _iterator3 = require('babel-runtime/core-js/symbol/iterator');

var _iterator4 = _interopRequireDefault(_iterator3);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _stream = require('../stream');

var _stream2 = _interopRequireDefault(_stream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var subscriptions = (0, _symbol2.default)('Message subscriptions');
var connections = (0, _symbol2.default)('Connections');

/**
 * Manages a group of network connections.
 *
 * @class ConnectionGroup
 */

var ConnectionGroup = function (_Emitter) {
  (0, _inherits3.default)(ConnectionGroup, _Emitter);

  function ConnectionGroup() {
    var _ref;

    var _temp, _this, _ret;

    (0, _classCallCheck3.default)(this, ConnectionGroup);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = ConnectionGroup.__proto__ || (0, _getPrototypeOf2.default)(ConnectionGroup)).call.apply(_ref, [this].concat(args))), _this), _this[subscriptions] = {}, _this[connections] = {}, _this.messages = new _stream2.default(function (push) {

      // Create a subscription.
      var subscribeToConnection = function subscribeToConnection(connection) {
        var subscription = connection.messages.forEach(push);
        _this[subscriptions][connection.id] = subscription;
      };

      // Dispose of the subscription.
      var disposeOfSubscription = function disposeOfSubscription(connection) {
        var subscription = _this[subscriptions][connection.id];
        subscription.dispose();
      };

      // Subscribe and forward all message events.
      _this.on('add', subscribeToConnection);
      _this.on('remove', disposeOfSubscription);
      [].concat((0, _toConsumableArray3.default)(_this)).forEach(subscribeToConnection);

      // Remove all listeners.
      return function () {
        _this.removeListener('add', subscribeToConnection);
        _this.removeListener('remove', disposeOfSubscription);

        [].concat((0, _toConsumableArray3.default)(_this)).forEach(disposeOfSubscription);
      };
    }), _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
  }

  /**
   * Creates a connection group from a connection,
   * @param  {ConnectionGroup|Object} connection -
   * Either a group of connections, or a single connection.
   * @return {ConnectionGroup} -
   * If a group is given, the same group is returned.
   * If a connection is given, then it's wrapped into a new group.
   */


  (0, _createClass3.default)(ConnectionGroup, [{
    key: 'id',


    /**
     * Retrieves the connection that matches the given ID.
     * @param  {String} key - Connection identifier.
     * @return {Object|null} - The matching connection.
     */
    value: function id(key) {
      return this[connections][key] || null;
    }

    /**
     * Adds a connection to the group.
     * @param  {Object} connection - Network connection.
     * @return {undefined}
     */

  }, {
    key: 'add',
    value: function add(connection) {
      var exists = this[connections].hasOwnProperty(connection.id);

      if (!exists) {
        this[connections][connection.id] = connection;
        this.emit('add', connection);
      }

      return connection;
    }

    /**
     * Removes a connection from the group.
     * @param  {Object} connection - The connection in question.
     * @return {undefined}
     */

  }, {
    key: 'remove',
    value: function remove(connection) {
      var exists = this[connections].hasOwnProperty(connection.id);

      if (exists) {
        delete this[connections][connection.id];
        this.emit('remove', connection);
      }

      return connection;
    }

    /**
     * Creates a new list containing only connections satisfying the predicate.
     * @param  {Function} predicate - Returns whether the value should be kept.
     * @return {ConnectionGroup} - The resulting set.
     */

  }, {
    key: 'filter',
    value: function filter(predicate) {
      var result = new ConnectionGroup();

      // Add matching items.
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var connection = _step.value;

          if (predicate(connection)) {
            result.add(connection);
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return result;
    }

    /**
     * Creates a list of connections of only the given type.
     * @param  {String} type - Connection type (e.g., 'websocket', 'http').
     * @return {ConnectionGroup} - A new list containing only those connections.
     */

  }, {
    key: 'type',
    value: function type(_type) {
      return this.filter(function (connection) {
        return connection.type === _type;
      });
    }

    /**
     * Broadcasts a message through every connection in the group.
     * @param  {Any} msg - A message to send.
     * @return {undefined}
     */

  }, {
    key: 'send',
    value: function send(msg) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(this), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var connection = _step2.value;

          connection.send(msg);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }

    /**
     * Returns whether the connection is contained within the group.
     * @param  {Object} connection - A network plugin.
     * @param  {String} connection.id - The connection's unique identifier.
     * @return {Boolean} - Whether it's contained.
     */

  }, {
    key: 'has',
    value: function has(connection) {
      return this[connections].hasOwnProperty(connection.id);
    }

    /**
     * Joins two groups into a new group.
     * @param  {ConnectionGroup} group - Source connections.
     * @return {ConnectionGroup} - The two groups joined together.
     */

  }, {
    key: 'union',
    value: function union(group) {
      var _this2 = this;

      var result = new ConnectionGroup();
      var addToGroup = result.add.bind(result);

      var removeFromGroup = function removeFromGroup(connection) {
        if (!_this2.has(connection) && !group.has(connection)) {
          result.remove(connection);
        }
      };

      [].concat((0, _toConsumableArray3.default)(group)).forEach(addToGroup);
      [].concat((0, _toConsumableArray3.default)(this)).forEach(addToGroup);

      group.on('add', addToGroup);
      this.on('add', addToGroup);

      group.on('remove', removeFromGroup);
      this.on('remove', removeFromGroup);

      return result;
    }

    /**
     * Yields every connection in the group.
     * @return {undefined}
     */

  }, {
    key: _iterator4.default,
    value: _regenerator2.default.mark(function value() {
      var group, id;
      return _regenerator2.default.wrap(function value$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              group = this[connections];
              _context.t0 = _regenerator2.default.keys(group);

            case 2:
              if ((_context.t1 = _context.t0()).done) {
                _context.next = 9;
                break;
              }

              id = _context.t1.value;

              if (!group.hasOwnProperty(id)) {
                _context.next = 7;
                break;
              }

              _context.next = 7;
              return group[id];

            case 7:
              _context.next = 2;
              break;

            case 9:
            case 'end':
              return _context.stop();
          }
        }
      }, value, this);
    })
  }]);
  return ConnectionGroup;
}(_eventemitter2.default);

ConnectionGroup.ensure = function (connection) {
  if (connection instanceof ConnectionGroup) {
    return connection;
  }

  var group = new ConnectionGroup();
  group.add(connection);

  return group;
};

exports.default = ConnectionGroup;