'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _index = require('../../index');

var _index2 = _interopRequireDefault(_index);

var _mocks = require('../../mocks');

var _graphCrdt = require('graph-crdt');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */
describe('Database hook', function () {
  var db = void 0,
      storage = void 0,
      hook = void 0;

  beforeEach(function () {
    storage = new _mocks.Storage();
    hook = (0, _expect.createSpy)().andCall(function (write) {
      return write;
    });
  });

  describe('"before.write"', function () {
    var write = void 0;

    beforeEach(function () {
      write = (0, _expect.spyOn)(storage, 'write').andCallThrough();
      db = (0, _index2.default)({
        storage: [storage],
        hooks: {
          before: { write: hook }
        }
      });
    });

    afterEach(function () {
      return write.restore();
    });

    it('should be called before writes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return db.write('users', {});

            case 2:
              (0, _expect2.default)(hook).toHaveBeenCalled();

            case 3:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    })));

    it('should be able to override the graph', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var graph, _write$calls$0$argume, state;

      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              graph = new _graphCrdt.Graph();

              hook.andCall(function (write) {
                return (0, _extends3.default)({}, write, { graph: graph });
              });

              _context2.next = 4;
              return db.write('stuff', {});

            case 4:
              _write$calls$0$argume = (0, _slicedToArray3.default)(write.calls[0].arguments, 1), state = _write$calls$0$argume[0].graph;

              (0, _expect2.default)(state).toBe(graph);

            case 6:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, undefined);
    })));

    it('should be able to prevent writes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
      var testError;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              testError = new Error('Testing prevented writes');

              hook.andCall();
              hook.andThrow(testError);

              _context3.prev = 3;
              _context3.next = 6;
              return db.write('user', { status: 'rejected (hopefully)' });

            case 6:
              throw new Error('Pipeline before.write rejection was ignored.');

            case 9:
              _context3.prev = 9;
              _context3.t0 = _context3['catch'](3);

              (0, _expect2.default)(_context3.t0).toBe(testError);

            case 12:

              (0, _expect2.default)(write).toNotHaveBeenCalled();
              _context3.next = 15;
              return db.read('user');

            case 15:
              _context3.t1 = _context3.sent;
              (0, _expect2.default)(_context3.t1).toBe(null);

            case 17:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, undefined, [[3, 9]]);
    })));
  });

  describe('"after.write"', function () {

    beforeEach(function () {
      db = (0, _index2.default)({
        hooks: {
          after: { write: hook }
        }
      });
    });

    it('should be called after writes', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return db.write('users', {});

            case 2:
              (0, _expect2.default)(hook).toHaveBeenCalled();

            case 3:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, undefined);
    })));

    it('should be passed the options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
      var _hook$calls$0$argumen, write;

      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return db.write('beans', {}, {
                cool: true
              });

            case 2:
              _hook$calls$0$argumen = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), write = _hook$calls$0$argumen[0];

              (0, _expect2.default)(write).toContain({
                cool: true
              });

            case 4:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined);
    })));
  });

  describe('"before.read.node"', function () {
    var read = void 0,
        hook = void 0;

    beforeEach(function () {
      read = (0, _expect.spyOn)(storage, 'read');
      hook = (0, _expect.createSpy)().andCall(function (read) {
        return read;
      });
      db = (0, _index2.default)({
        storage: [storage],
        hooks: {
          before: {
            read: { node: hook }
          }
        }
      });
    });

    it('should call the hook before reads', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              (0, _expect2.default)(hook).toNotHaveBeenCalled();
              _context6.next = 3;
              return db.read('users');

            case 3:
              (0, _expect2.default)(hook).toHaveBeenCalled();

            case 4:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, undefined);
    })));

    it('should pass the key and options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
      var options, _hook$calls$0$argumen2, read;

      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              options = { 'engage hyperdrive': true };
              _context7.next = 3;
              return db.read('key name', options);

            case 3:
              _hook$calls$0$argumen2 = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), read = _hook$calls$0$argumen2[0];

              (0, _expect2.default)(read.key).toBe('key name');
              (0, _expect2.default)(read).toContain(options);

            case 6:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined);
    })));

    it('should allow overriding the key', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      var _read$calls$0$argumen, key;

      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  key: 'prefix/' + read.key
                });
              });
              _context8.next = 3;
              return db.read('stuff');

            case 3:
              _read$calls$0$argumen = (0, _slicedToArray3.default)(read.calls[0].arguments, 1), key = _read$calls$0$argumen[0].key;

              (0, _expect2.default)(key).toBe('prefix/stuff');

            case 5:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));

    it('should allow overriding the options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee9() {
      var _read$calls$0$argumen2, options;

      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  overridden: true
                });
              });

              _context9.next = 3;
              return db.read('key', { original: true });

            case 3:
              _read$calls$0$argumen2 = (0, _slicedToArray3.default)(read.calls[0].arguments, 1), options = _read$calls$0$argumen2[0];

              (0, _expect2.default)(options).toContain({
                overridden: true,
                original: true
              });

            case 5:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined);
    })));
  });

  describe('"after.read.node"', function () {
    var hook = void 0;

    beforeEach((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee10() {
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (read) {
                return read;
              });
              db = (0, _index2.default)({
                hooks: {
                  after: {
                    read: { node: hook }
                  }
                }
              });

              _context10.next = 4;
              return db.write('data', {
                existing: true
              });

            case 4:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, undefined);
    })));

    it('should be called after reads', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee11() {
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              _context11.next = 2;
              return db.read('data');

            case 2:
              (0, _expect2.default)(hook).toHaveBeenCalled();

            case 3:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, undefined);
    })));

    it('should be passed the read value and options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee12() {
      var data, _hook$calls$0$argumen3, read;

      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _context12.next = 2;
              return db.read('data', { cool: 'totally' });

            case 2:
              data = _context12.sent;
              _hook$calls$0$argumen3 = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), read = _hook$calls$0$argumen3[0];


              (0, _expect2.default)(read.context).toBe(data);
              (0, _expect2.default)(read).toContain({ cool: 'totally' });

            case 6:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, undefined);
    })));

    it('should allow override of the return node', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee13() {
      var value;
      return _regenerator2.default.wrap(function _callee13$(_context13) {
        while (1) {
          switch (_context13.prev = _context13.next) {
            case 0:
              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  context: 'haha, replaced!'
                });
              });
              _context13.next = 3;
              return db.read('data');

            case 3:
              value = _context13.sent;

              (0, _expect2.default)(value).toBe('haha, replaced!');

            case 5:
            case 'end':
              return _context13.stop();
          }
        }
      }, _callee13, undefined);
    })));
  });

  describe('"before.read.field"', function () {
    var hook = void 0,
        node = void 0,
        read = void 0;

    beforeEach((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee14() {
      return _regenerator2.default.wrap(function _callee14$(_context14) {
        while (1) {
          switch (_context14.prev = _context14.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (read) {
                return read;
              });
              db = (0, _index2.default)({
                storage: [storage],
                hooks: {
                  before: {
                    read: { field: hook }
                  }
                }
              });

              _context14.next = 4;
              return db.write('context hook test', {});

            case 4:
              node = _context14.sent;

              read = (0, _expect.spyOn)(storage, 'read').andCallThrough();

            case 6:
            case 'end':
              return _context14.stop();
          }
        }
      }, _callee14, undefined);
    })));

    it('should be passed the field, node, and options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee15() {
      var _hook$calls$0$argumen4, read;

      return _regenerator2.default.wrap(function _callee15$(_context15) {
        while (1) {
          switch (_context15.prev = _context15.next) {
            case 0:
              _context15.next = 2;
              return node.read('name', { passed: true });

            case 2:
              (0, _expect2.default)(hook).toHaveBeenCalled();

              _hook$calls$0$argumen4 = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), read = _hook$calls$0$argumen4[0];


              (0, _expect2.default)(read.node).toBe(node);
              (0, _expect2.default)(read.field).toBe('name');
              (0, _expect2.default)(read).toContain({ passed: true });

            case 7:
            case 'end':
              return _context15.stop();
          }
        }
      }, _callee15, undefined);
    })));

    it('should allow override of the field', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee16() {
      var value;
      return _regenerator2.default.wrap(function _callee16$(_context16) {
        while (1) {
          switch (_context16.prev = _context16.next) {
            case 0:
              _context16.next = 2;
              return node.write('replaced', 'resolved');

            case 2:
              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  field: 'replaced'
                });
              });

              _context16.next = 5;
              return node.read('nothing here');

            case 5:
              value = _context16.sent;

              (0, _expect2.default)(value).toBe('resolved');

            case 7:
            case 'end':
              return _context16.stop();
          }
        }
      }, _callee16, undefined);
    })));

    it('should allow override of the options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee17() {
      var _read$calls$0$argumen3, options;

      return _regenerator2.default.wrap(function _callee17$(_context17) {
        while (1) {
          switch (_context17.prev = _context17.next) {
            case 0:
              _context17.next = 2;
              return node.write('edge', { edge: 'potatoes' });

            case 2:

              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  overridden: true
                });
              });

              _context17.next = 5;
              return node.read('edge');

            case 5:
              _read$calls$0$argumen3 = (0, _slicedToArray3.default)(read.calls[0].arguments, 1), options = _read$calls$0$argumen3[0];

              (0, _expect2.default)(options).toContain({ overridden: true });

            case 7:
            case 'end':
              return _context17.stop();
          }
        }
      }, _callee17, undefined);
    })));
  });

  describe('"after.read.field"', function () {
    var hook = void 0,
        node = void 0;

    beforeEach((0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee18() {
      return _regenerator2.default.wrap(function _callee18$(_context18) {
        while (1) {
          switch (_context18.prev = _context18.next) {
            case 0:
              hook = (0, _expect.createSpy)().andCall(function (read) {
                return read;
              });
              db = (0, _index2.default)({
                hooks: {
                  after: {
                    read: { field: hook }
                  }
                }
              });

              _context18.next = 4;
              return db.write('after.read.field tests', {});

            case 4:
              node = _context18.sent;

            case 5:
            case 'end':
              return _context18.stop();
          }
        }
      }, _callee18, undefined);
    })));

    it('should be given the field, value, & options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee19() {
      var _hook$calls$0$argumen5, read;

      return _regenerator2.default.wrap(function _callee19$(_context19) {
        while (1) {
          switch (_context19.prev = _context19.next) {
            case 0:
              _context19.next = 2;
              return node.write('level', 5);

            case 2:
              _context19.next = 4;
              return node.read('level', { opts: true });

            case 4:
              (0, _expect2.default)(hook).toHaveBeenCalled();

              _hook$calls$0$argumen5 = (0, _slicedToArray3.default)(hook.calls[0].arguments, 1), read = _hook$calls$0$argumen5[0];


              (0, _expect2.default)(read).toContain({
                field: 'level',
                opts: true,
                value: 5
              });

            case 7:
            case 'end':
              return _context19.stop();
          }
        }
      }, _callee19, undefined);
    })));

    it('should allow overriding the value', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee20() {
      var value;
      return _regenerator2.default.wrap(function _callee20$(_context20) {
        while (1) {
          switch (_context20.prev = _context20.next) {
            case 0:
              hook.andCall(function (read) {
                return (0, _extends3.default)({}, read, {
                  value: 'replaced'
                });
              });

              _context20.next = 3;
              return node.read('whatever');

            case 3:
              value = _context20.sent;


              (0, _expect2.default)(value).toBe('replaced');

            case 5:
            case 'end':
              return _context20.stop();
          }
        }
      }, _callee20, undefined);
    })));

    it('should be called after resolving edges', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee21() {
      var edge, value;
      return _regenerator2.default.wrap(function _callee21$(_context21) {
        while (1) {
          switch (_context21.prev = _context21.next) {
            case 0:
              _context21.next = 2;
              return db.write('edge', {});

            case 2:
              edge = _context21.sent;
              _context21.next = 5;
              return node.write('edge', edge);

            case 5:

              hook.andCall(function (read) {
                (0, _expect2.default)(read.value).toBe(edge);

                return (0, _extends3.default)({}, read, {
                  value: 'replaced value'
                });
              });

              _context21.next = 8;
              return node.read('edge');

            case 8:
              value = _context21.sent;

              (0, _expect2.default)(value).toBe('replaced value');

            case 10:
            case 'end':
              return _context21.stop();
          }
        }
      }, _callee21, undefined);
    })));
  });
});