'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var symbol = {
  isOpen: (0, _symbol2.default)('Whether the stream is open'),
  callbacks: (0, _symbol2.default)('Event callback list'),
  dispose: (0, _symbol2.default)('Free stream resources'),
  observe: (0, _symbol2.default)('Force the stream open'),
  resolve: (0, _symbol2.default)('Completes the stream'),
  openStream: (0, _symbol2.default)('Stream publisher'),
  complete: (0, _symbol2.default)('Closes the stream'),
  handler: (0, _symbol2.default)('Stream callback'),
  push: (0, _symbol2.default)('Publish a message')
};

/**
 * References a callback subscription.
 * @class Subscription
 */

var Subscription = function () {

  /**
   * @param  {Stream} stream - Current stream context.
   * @param  {Function} callback - The subscribed function.
   */
  function Subscription(stream, callback) {
    (0, _classCallCheck3.default)(this, Subscription);

    this[symbol.handler] = callback;
    this.stream = stream;
  }

  /**
   * Unsubscribes the callback.
   * @return {undefined}
   */


  (0, _createClass3.default)(Subscription, [{
    key: 'dispose',
    value: function dispose() {
      var _this = this;

      var stream = this.stream;

      // Remove the listener.

      var filtered = this.stream[symbol.callbacks].filter(function (callback) {
        return callback !== _this[symbol.handler];
      });

      stream[symbol.callbacks] = filtered;

      // The last listener unsubscribed.
      if (!filtered.length) {
        stream[symbol.isOpen] = false;

        if (stream[symbol.dispose]) {
          stream[symbol.dispose]();
        }
      }
    }
  }]);
  return Subscription;
}();

/**
 * Lazy, asynchronous, cacheless streams.
 * @class Stream
 */


var Stream = function () {
  (0, _createClass3.default)(Stream, null, [{
    key: 'create',


    /**
     * Constructs a new Stream instance, `new` optional.
     * @param  {Function} publisher - Stream publisher.
     * @return {Stream} - The new stream instance.
     */
    value: function create(publisher) {
      return new Stream(publisher);
    }

    /**
     * Creates a stream from an event source.
     * @param  {EventEmitter} emitter
     * Really anything that looks like an event emitter.
     * @param  {String} event - The event name.
     * @return {Stream} - An event stream.
     */

  }, {
    key: 'fromEvent',
    value: function fromEvent(emitter, event) {
      return new Stream(function (push) {
        emitter.on(event, push);

        return function () {
          return emitter.removeListener(event, push);
        };
      });
    }
  }]);

  /**
   * @param {Function} publisher - Used to generate consumable data.
   */
  function Stream(publisher) {
    var _this2 = this;

    (0, _classCallCheck3.default)(this, Stream);
    this.complete = new _promise2.default(function (res) {
      _this2[symbol.resolve] = res;
    });

    this[symbol.push] = function (msg) {
      _this2[symbol.callbacks].forEach(function (callback) {
        return callback(msg);
      });
    };

    this[symbol.complete] = function () {
      _this2[symbol.resolve]();
    };

    if (!(publisher instanceof Function)) {
      throw new TypeError('Stream(...) expects a function, was given "' + publisher + '"');
    }

    this[symbol.openStream] = publisher;
    this[symbol.callbacks] = [];
  }

  /**
   * Subscribes to the event stream.
   * @param  {Function} callback - Invoked for every event.
   * @return {stream} - The current stream.
   */


  (0, _createClass3.default)(Stream, [{
    key: 'forEach',
    value: function forEach(callback) {
      this[symbol.callbacks].push(callback);

      this[symbol.observe]();

      return new Subscription(this, callback);
    }

    /**
     * Pushes a new message.
     * @private
     * @param  {any} msg - A message in the stream.
     * @return {undefined}
     */


    /**
     * Terminates the stream permanently.
     * @return {undefined}
     */

  }, {
    key: symbol.observe,


    /**
     * Forces the stream open.
     * @private
     * @return {undefined}
     */
    value: function value() {
      if (this[symbol.isOpen]) {
        return;
      }

      this[symbol.dispose] = this[symbol.openStream](this[symbol.push], this[symbol.complete]);

      this[symbol.isOpen] = true;
    }
  }]);
  return Stream;
}();

exports.default = Stream;